# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from datetime import timedelta, date
from workalendar.core import WesternCalendar
from workalendar.core import SUN
from workalendar.core import IslamicMixin, ChristianMixin


class Algeria(WesternCalendar, IslamicMixin):
    "Algeria"
    # Islamic holidays
    include_prophet_birthday = True
    include_eid_al_fitr = True
    include_day_of_sacrifice = True
    include_islamic_new_year = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (7, 5, "Independence Day"),
        (11, 1, "Anniversary of the revolution"),
    )

    ISLAMIC_HOLIDAYS = IslamicMixin.ISLAMIC_HOLIDAYS + (
        (1, 10, "Ashura"),
    )


class Benin(WesternCalendar, IslamicMixin, ChristianMixin):
    "Benin"
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True
    # Islamic holidays
    include_prophet_birthday = True
    include_eid_al_fitr = True
    include_day_of_sacrifice = True
    include_day_of_sacrifice_label = "Tabaski"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 10, "Traditional Day"),
        (5, 1, "Labour Day"),
        (8, 1, "Independence Day"),
        (10, 26, "Armed Forces Day"),
        (11, 30, "National Day"),
    )


class IvoryCoast(WesternCalendar, ChristianMixin, IslamicMixin):
    "Ivory Coast"
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True
    include_day_after_prophet_birthday = True
    include_eid_al_fitr = True
    include_day_of_sacrifice = True
    include_day_of_sacrifice_label = "Feast of the Sacrifice"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (8, 7, "Independence Day"),
        (11, 15, "National Peace Day"),
    )


class Madagascar(WesternCalendar, ChristianMixin):
    "Madagascar"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 29, "Martyrs' Day"),
        (5, 1, "Labour Day"),
        (6, 26, "Independence Day"),
    )
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True


class SaoTomeAndPrincipe(WesternCalendar, ChristianMixin):
    "São Tomé and Príncipe"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (2, 3, "Martyr's Day"),
        (5, 1, "Labour Day"),
        (7, 12, "Independence Day"),
        (9, 6, "Armed Forces Day"),
        (9, 30, "Agricultural Reform Day"),
        (12, 21, u"São Tomé Day"),
    )
    include_all_saints = True


class SouthAfrica(WesternCalendar, ChristianMixin):
    "South Africa"
    include_good_friday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 21, 'Human Rights Day'),
        (4, 27, "Freedom Day"),
        (5, 1, "Workers Day"),
        (6, 16, "Youth Day"),
        (8, 9, "National Women Day"),
        (9, 24, "Heritage Day"),
        (12, 16, "Day of reconcilation"),
        (12, 26, "Day of good will"),
    )

    def get_family_day(self, year):
        return (self.get_good_friday(year), "Family Day")

    def get_variable_days(self, year):
        days = super(SouthAfrica, self).get_variable_days(year)
        days.append(self.get_family_day(year))
        # compute shifting days
        for month, day, label in self.FIXED_HOLIDAYS:
            holiday = date(year, month, day)
            if holiday.weekday() == SUN:
                days.append((
                    holiday + timedelta(days=1),
                    "%s substitute" % label
                ))
        return days



# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from datetime import date, timedelta
from workalendar.core import WesternCalendar, ChristianMixin
from workalendar.core import SUN, MON, TUE, WED, FRI, SAT


class Brazil(WesternCalendar, ChristianMixin):
    "Brazil"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (4, 21, "Tiradentes' Day"),
        (5, 1, "Labour Day"),
        (9, 7, "Independence Day"),
        (10, 12, "Our Lady of Aparecida"),
        (11, 2, "All Souls' Day"),
        (11, 15, "Republic Day"),
    )


class BrazilSaoPauloState(Brazil):
    "Brazil São Paulo State"
    FIXED_HOLIDAYS = Brazil.FIXED_HOLIDAYS + (
        (7, 9, "Constitutional Revolution of 1932"),
    )


class BrazilSaoPauloCity(BrazilSaoPauloState):
    "Brazil São Paulo City"
    FIXED_HOLIDAYS = BrazilSaoPauloState.FIXED_HOLIDAYS + (
        (1, 25, "Anniversary of the city of São Paulo"),
        (11, 20, "Dia da Consciência Negra")
    )
    include_easter_sunday = True
    include_corpus_christi = True

    def get_carnaval(self, year):
        return self.get_easter_sunday(year) - timedelta(days=47)

    def get_variable_days(self, year):
        days = super(BrazilSaoPauloCity, self).get_variable_days(year)
        days.append((self.get_carnaval(year), "Carnaval"))
        days.append((self.get_good_friday(year), "Sexta-feira da Paixão"))
        return days


class Chile(WesternCalendar, ChristianMixin):
    "Chile"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (5, 21, "Navy Day"),
        (6, 29, "Saint Peter and Saint Paul"),
        (7, 16, "Our Lady of Mount Carmel"),
        (9, 18, "National holiday"),
        (9, 19, "Army holiday"),
        (10, 12, "Columbus Day"),
        (12, 31, "Banking Holiday"),
    )
    include_good_friday = True
    include_easter_saturday = True
    include_assumption = True
    include_all_saints = True
    include_immaculate_conception = True

    def get_variable_days(self, year):
        days = super(Chile, self).get_variable_days(year)
        september_17 = date(year, 9, 17)
        if september_17.weekday() == MON:
            days.append((september_17, '"Bridge" holiday'))
        september_20 = date(year, 9, 20)
        if september_20.weekday() == FRI:
            days.append((september_20, '"Bridge" holiday'))

        reformation_day = date(year, 10, 31)
        if reformation_day.weekday() == WED:
            reformation_day = date(year, 11, 2)
        elif reformation_day.weekday() == TUE:
            reformation_day = date(year, 10, 27)

        days.append((reformation_day, "Reformation Day"))

        return days


class Colombia(WesternCalendar, ChristianMixin):
    "Colombia"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (7, 20, "Independence Day"),
        (8, 7, "Boyacá Battle"),
    )
    include_palm_sunday = True
    include_holy_thursday = True
    include_good_friday = True
    include_easter_sunday = True
    include_corpus_christi = True
    include_immaculate_conception = True

    def get_epiphany(self, year):
        base_day = date(year, 1, 6)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_saint_joseph(self, year):
        base_day = date(year, 3, 19)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_ascension(self, year):
        return self.get_easter_sunday(year) + timedelta(days=43)

    def get_corpus_christi(self, year):
        return self.get_easter_sunday(year) + timedelta(days=64)

    def get_sacred_heart(self, year):
        return self.get_easter_sunday(year) + timedelta(days=71)

    def get_saint_peter_and_saint_paul(self, year):
        base_day = date(year, 6, 29)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_assumption(self, year):
        base_day = date(year, 8, 15)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_race_day(self, year):
        base_day = date(year, 10, 12)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_all_saints(self, year):
        base_day = date(year, 11, 1)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_cartagena_independence(self, year):
        base_day = date(year, 11, 11)
        return Colombia.get_first_weekday_after(base_day, 0)

    def get_variable_days(self, year):
        days = super(Colombia, self).get_variable_days(year)
        days.extend([
            (self.get_epiphany(year), "Epiphany"),
            (self.get_saint_joseph(year), "Saint Joseph"),
            (self.get_ascension(year), "Ascension"),
            (self.get_sacred_heart(year), "Sacred Heart"),
            (self.get_saint_peter_and_saint_paul(year),
                "Saint Peter and Saint Paul"),
            (self.get_assumption(year), "Assumption of Mary to Heaven"),
            (self.get_race_day(year), "Race Day"),
            (self.get_all_saints(year), "All Saints"),
            (self.get_cartagena_independence(year),
                "Cartagena's Independence"),
        ])

        return days


class Mexico(WesternCalendar, ChristianMixin):
    "Mexico"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (9, 16, "Independence Day"),
    )

    def get_variable_days(self, year):
        days = super(Mexico, self).get_variable_days(year)
        days.append(
            (Mexico.get_nth_weekday_in_month(year, 2, MON),
             "Constitution Day"))

        days.append(
            (Mexico.get_nth_weekday_in_month(year, 3, MON, 3),
             "Benito Juárez's birthday"))

        days.append(
            (Mexico.get_nth_weekday_in_month(year, 11, MON, 3),
             "Revolution Day"))

        return days

    def get_calendar_holidays(self, year):
        days = super(Mexico, self).get_calendar_holidays(year)
        # If any statutory day is on Sunday, the monday is off
        # If it's on a Saturday, the Friday is off
        for day, label in days:
            if day.weekday() == SAT:
                days.append((day - timedelta(days=1), "%s substitute" % label))
            elif day.weekday() == SUN:
                days.append((day + timedelta(days=1), "%s substitute" % label))
        # Extra: if new year's day is a saturday, the friday before is off
        next_new_year = date(year + 1, 1, 1)
        if next_new_year.weekday():
            days.append((date(year, 12, 31), "New Year Day substitute"))
        return days


class Panama(WesternCalendar, ChristianMixin):
    "Panama"
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 9, "Martyrs' Day"),
        (5, 1, "Labour Day"),
        (11, 3, "Independence Day"),
        (11, 5, "Colon Day"),
        (11, 10, "Shout in Villa de los Santos"),
        (12, 2, "Independence from Spain"),
        (12, 8, "Mothers' Day"),
    )

    def get_variable_days(self, year):
        days = super(Panama, self).get_variable_days(year)
        days.append(
            (self.get_ash_wednesday(year) - timedelta(days=1), "Carnival")
        )
        return days



from workalendar.core import WesternCalendar, ChristianMixin
from workalendar.core import MON, TUE, FRI
from datetime import date, timedelta


class Australia(WesternCalendar, ChristianMixin):
    "Australia"
    include_good_friday = True
    include_easter_monday = True
    include_queens_birthday = False
    include_labour_day_october = False
    include_boxing_day = True
    # Shall we shift Anzac Day?
    shift_anzac_day = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 26, "Australia Day"),
    )

    def get_canberra_day(self, year):
        return (
            Australia.get_nth_weekday_in_month(year, 3, MON, 2),
            "Canberra Day"
        )

    def get_queens_birthday(self, year):
        return (
            Australia.get_nth_weekday_in_month(year, 6, MON, 2),
            "Queen's Birthday"
        )

    def get_labour_day_october(self, year):
        return (
            Australia.get_nth_weekday_in_month(year, 10, MON),
            'Labour Day'
        )

    def get_anzac_day(self, year):
        anzac_day = date(year, 4, 25)
        if not self.shift_anzac_day:
            return (anzac_day, "Anzac Day")
        if anzac_day.weekday() in self.get_weekend_days():
            anzac_day = self.find_following_working_day(anzac_day)
        return (anzac_day, "Anzac Day")

    def get_variable_days(self, year):
        # usual variable days
        days = super(Australia, self).get_variable_days(year)
        january_first = date(year, 1, 1)
        if january_first.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(january_first),
                "New Year's Day shift")
            )

        australia_day = date(year, 1, 26)
        if australia_day.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(australia_day),
                "Australia Day shift")
            )

        # was fixed, but might be shifted
        days.append(self.get_anzac_day(year))

        if self.include_queens_birthday:
            days.append(self.get_queens_birthday(year))
        if self.include_labour_day_october:
            days.append(self.get_labour_day_october(year))

        christmas = date(year, 12, 25)
        boxing_day = date(year, 12, 26)
        if christmas.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(christmas)
            days.append((shift, "Christmas Shift"))
            days.append((shift + timedelta(days=1), "Boxing Day Shift"))
        elif boxing_day.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(boxing_day)
            days.append((shift, "Boxing Day Shift"))

        return days


class AustraliaCapitalTerritory(Australia):
    "Australia Capital Territory"
    include_easter_saturday = True
    include_queens_birthday = True
    include_labour_day_october = True
    include_boxing_day = True

    def get_family_community_day(self, year):
        # Since this day is picked unsing the school year calendar, there's no
        # mathematical way yet to provide it surely

        # Family & Community Day was celebrated on the first Tuesday of
        # November in 2007, 2008 and 2009
        if year in (2007, 2008, 2009):
            day = AustraliaCapitalTerritory.get_nth_weekday_in_month(
                year, 11, TUE)
        elif year == 2010:
            day = date(2010, 9, 27)
        elif year == 2011:
            day = date(2011, 10, 10)
        elif year == 2012:
            day = date(2012, 10, 8)
        elif year == 2013:
            day = date(2013, 9, 30)
        elif year == 2014:
            day = date(2014, 9, 29)
        elif year == 2015:
            day = date(2015, 9, 28)
        elif year == 2016:
            day = date(2016, 9, 26)
        else:
            raise Exception("Year %d is not implemented, Sorry" % year)
        return (day, "Family & Community Day")

    def get_variable_days(self, year):
        days = super(AustraliaCapitalTerritory, self) \
            .get_variable_days(year)
        days += [
            self.get_canberra_day(year),
            self.get_family_community_day(year),
        ]
        return days


class AustraliaNewSouthWales(Australia):
    "Australia New South Wales"
    include_queens_birthday = True
    include_easter_saturday = True
    include_easter_sunday = True
    include_labour_day_october = True
    include_boxing_day = True
    shift_anzac_day = False


class AustraliaNorthernTerritory(Australia):
    "Australia Northern Territory"
    include_easter_saturday = True
    include_queens_birthday = True
    include_boxing_day = True

    def get_may_day(self, year):
        return (
            AustraliaNorthernTerritory.get_nth_weekday_in_month(
                year, 5, MON),
            "May Day"
        )

    def get_picnic_day(self, year):
        return (
            AustraliaNorthernTerritory.get_nth_weekday_in_month(
                year, 8, MON),
            "Picnic Day"
        )

    def get_variable_days(self, year):
        days = super(AustraliaNorthernTerritory, self) \
            .get_variable_days(year)
        days += [
            self.get_may_day(year),
            self.get_picnic_day(year),
        ]
        return days


class AustraliaQueensland(Australia):
    "Australia Queensland"
    include_easter_saturday = True
    include_queens_birthday = True
    include_boxing_day = True

    def get_labour_day_may(self, year):
        return (
            AustraliaNorthernTerritory.get_nth_weekday_in_month(
                year, 5, MON),
            "Labour Day"
        )

    def get_variable_days(self, year):
        days = super(AustraliaQueensland, self) \
            .get_variable_days(year)
        days += [
            self.get_labour_day_may(year),
        ]
        return days


class SouthAustralia(Australia):
    "South Australia"
    include_easter_saturday = True
    include_queens_birthday = True
    include_labour_day_october = True

    def get_adelaides_cup(self, year):
        return (
            SouthAustralia.get_nth_weekday_in_month(
                year, 3, MON, 2),
            "Adelaide's cup"
        )

    def get_proclamation_day(self, year):
        return (date(year, 12, 26), "Proclamation Day")

    def get_variable_days(self, year):
        days = super(SouthAustralia, self) \
            .get_variable_days(year)
        days += [
            self.get_adelaides_cup(year),
            self.get_proclamation_day(year),
        ]
        return days


class Tasmania(Australia):
    "Tasmania"
    include_queens_birthday = True
    include_boxing_day = True
    shift_anzac_day = False

    @property
    def has_recreation_day(self):
        return True

    def get_eight_hours_day(self, year):
        return (
            Tasmania.get_nth_weekday_in_month(year, 3, MON, 2),
            "Eight hours Day"
        )

    def get_recreation_day(self, year):
        return (
            Tasmania.get_nth_weekday_in_month(year, 11, MON),
            "Recreation Day"
        )

    def get_variable_days(self, year):
        days = super(Tasmania, self).get_variable_days(year)
        days.append(self.get_eight_hours_day(year))
        if self.has_recreation_day:
            days.append(self.get_recreation_day(year))
        return days


class Hobart(Tasmania):
    "Hobart"
    @property
    def has_recreation_day(self):
        return False

    def get_hobart(self, year):
        return (
            Hobart.get_nth_weekday_in_month(year, 2, MON, 2),
            "Royal Hobart Regatta"
        )

    def get_variable_days(self, year):
        days = super(Hobart, self).get_variable_days(year)
        days.append(self.get_hobart(year))
        return days


class Victoria(Australia):
    "Victoria"
    include_easter_saturday = True
    include_queens_birthday = True
    include_boxing_day = True

    def get_labours_day_in_march(self, year):
        return (
            Victoria.get_nth_weekday_in_month(year, 3, MON, 2),
            "Labour Day"
        )

    def get_melbourne_cup(self, year):
        return (
            Victoria.get_nth_weekday_in_month(year, 11, TUE),
            "Melbourne Cup"
        )

    def get_variable_days(self, year):
        days = super(Victoria, self).get_variable_days(year)
        days.append(self.get_labours_day_in_march(year))
        days.append(self.get_melbourne_cup(year))
        return days


class WesternAustralia(Australia):
    "Western Australia"
    include_boxing_day = True

    def get_labours_day_in_march(self, year):
        return (
            WesternAustralia.get_nth_weekday_in_month(year, 3, MON),
            "Labour Day"
        )

    def get_western_australia_day(self, year):
        return (
            WesternAustralia.get_nth_weekday_in_month(year, 6, MON),
            "Western Australia Day"
        )

    def get_variable_days(self, year):
        # It is not possible to surely compute Queen's Birthday holiday in
        # The western Australia territory, since it's based on the Governor
        # Decision (it is typically the last Monday of September or the first
        # Monday of October)

        days = super(WesternAustralia, self).get_variable_days(year)
        days.append(self.get_labours_day_in_march(year))
        days.append(self.get_western_australia_day(year))
        return days


class MarshallIslands(WesternCalendar, ChristianMixin):
    "Marshall Islands"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 3, "Remembrance Day"),
        (5, 1, "Constitution Day"),
        (11, 17, "Presidents' Day"),
        (12, 31, "New Year's Eve"),
    )
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(MarshallIslands, self).get_variable_days(year)
        days.append((
            MarshallIslands.get_nth_weekday_in_month(year, 7, FRI),
            "Fishermen's Holiday"
        ))
        days.append((
            MarshallIslands.get_nth_weekday_in_month(year, 9, FRI),
            "Labour Day"
        ))
        days.append((
            MarshallIslands.get_last_weekday_in_month(year, 9, FRI),
            "Manit Day"
        ))
        days.append((
            MarshallIslands.get_nth_weekday_in_month(year, 12, FRI),
            "Gospel Day"
        ))
        return days



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
from os.path import join, dirname, abspath
import sys

PY2 = sys.version_info[0] == 2

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup  # NOQA


def read_relative_file(filename):
    """Returns contents of the given file, whose path is supposed relative
    to this module."""
    path = join(dirname(abspath(__file__)), filename)
    with io.open(path, encoding='utf-8') as f:
        return f.read()

NAME = 'workalendar'
DESCRIPTION = 'Worldwide holidays and working days helper and toolkit.'
REQUIREMENTS = [
    'python-dateutil',
    'lunardate',
    'pytz',
    'pyCalverter',
]
version = '0.4.5'
__VERSION__ = version

if PY2:
    REQUIREMENTS.append('pyephem')
else:
    REQUIREMENTS.append('ephem')

params = dict(
    name=NAME,
    description=DESCRIPTION,
    packages=['workalendar'],
    version=__VERSION__,
    long_description=read_relative_file('README.rst'),
    author='Bruno Bord',
    author_email='bruno.bord@people-doc.com',
    url='https://github.com/novafloss/workalendar',
    license='MIT License',
    include_package_data=True,
    install_requires=REQUIREMENTS,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
)

if __name__ == '__main__':
    setup(**params)



"""Working day tools
"""
import warnings
from calendar import monthrange
from datetime import date, timedelta, datetime
from math import pi

import ephem
import pytz
from calverter import Calverter
from dateutil import easter
from lunardate import LunarDate

MON, TUE, WED, THU, FRI, SAT, SUN = range(7)


class Calendar(object):

    FIXED_HOLIDAYS = ()

    def __init__(self):
        self._holidays = {}

    def get_fixed_holidays(self, year):
        """Return the fixed days according to the FIXED_HOLIDAYS class property
        """
        days = []
        for month, day, label in self.FIXED_HOLIDAYS:
            days.append((date(year, month, day), label))
        return days

    def get_variable_days(self, year):
        return []

    def get_calendar_holidays(self, year):
        """Get calendar holidays.
        If you want to override this, please make sure that it **must** return
        a list of tuples (date, holiday_name)."""
        return self.get_fixed_holidays(year) + self.get_variable_days(year)

    def holidays(self, year=None):
        """Computes holidays (non-working days) for a given year.
        Return a 2-item tuple, composed of the date and a label."""
        if not year:
            year = date.today().year

        if year in self._holidays:
            return self._holidays[year]

        # Here we process the holiday specific calendar
        temp_calendar = tuple(self.get_calendar_holidays(year))

        # it is sorted
        self._holidays[year] = sorted(temp_calendar)
        return self._holidays[year]

    def get_holiday_label(self, day):
        """Return the label of the holiday, if the date is a holiday"""
        # a little exception: chop the datetime type
        if type(day) is datetime:
            day = day.date()
        return {day: label for day, label in self.holidays(day.year)
                }.get(day)

    def holidays_set(self, year=None):
        "Return a quick date index (set)"
        return set([day for day, label in self.holidays(year)])

    def get_weekend_days(self):
        """Return a list (or a tuple) of weekdays that are *not* working days.

        e.g: return (SAT, SUN,)

        """
        raise NotImplementedError("Your Calendar class must implement the"
                                  " `get_weekend_days` method")

    def is_working_day(self, day,
                       extra_working_days=None, extra_holidays=None):
        """Return True if it's a working day.
        In addition to the regular holidays, you can add exceptions.

        By providing ``extra_working_days``, you'll state that these dates
        **are** working days.

        By providing ``extra_holidays``, you'll state that these dates **are**
        holidays, even if not in the regular calendar holidays (or weekends).

        Please note that the ``extra_working_days`` list has priority over the
        ``extra_holidays`` list.

        """
        # a little exception: chop the datetime type
        if type(day) is datetime:
            day = day.date()

        # Extra lists exceptions
        if extra_working_days and day in extra_working_days:
            return True

        # Regular rules
        if day.weekday() in self.get_weekend_days():
            return False

        return not self.is_holiday(day, extra_holidays=extra_holidays)

    def is_holiday(self, day, extra_holidays=None):
        """Return True if it's an holiday.
        In addition to the regular holidays, you can add exceptions.

        By providing ``extra_holidays``, you'll state that these dates **are**
        holidays, even if not in the regular calendar holidays (or weekends).

        """
        # a little exception: chop the datetime type
        if type(day) is datetime:
            day = day.date()

        if extra_holidays and day in extra_holidays:
            return True

        return day in self.holidays_set(day.year)

    def add_working_days(self, day, delta,
                         extra_working_days=None, extra_holidays=None):
        """Add `delta` working days to the date.

        the ``delta`` parameter might be positive or negative. If it's
        negative, you may want to use the ``sub_working_days()`` method with
        a positive ``delta`` argument.

        By providing ``extra_working_days``, you'll state that these dates
        **are** working days.

        By providing ``extra_holidays``, you'll state that these dates **are**
        holidays, even if not in the regular calendar holidays (or weekends).

        Please note that the ``extra_working_days`` list has priority over the
        ``extra_holidays`` list.
        """
        days = 0
        temp_day = day
        day_added = 1 if delta >= 0 else -1
        delta = abs(delta)
        while days < delta:
            temp_day = temp_day + timedelta(days=day_added)
            if self.is_working_day(temp_day,
                                   extra_working_days=extra_working_days,
                                   extra_holidays=extra_holidays):
                days += 1
        return temp_day

    def sub_working_days(self, day, delta,
                         extra_working_days=None, extra_holidays=None):
        """
        Substract `delta` working days to the date.

        This method is a shortcut / helper. Users may want to use either::

            cal.add_working_days(my_date, -7)
            cal.sub_working_days(my_date, 7)

        The other parameters are to be used exactly as in the
        ``add_working_days`` method.

        A negative ``delta`` argument will be converted into its absolute
        value. Hence, the two following calls are equivalent::

            cal.sub_working_days(my_date, -7)
            cal.sub_working_days(my_date, 7)

        """
        delta = abs(delta)
        return self.add_working_days(
            day, -delta, extra_working_days, extra_holidays)

    def find_following_working_day(self, day):
        "Looks for the following working day"
        while day.weekday() in self.get_weekend_days():
            day = day + timedelta(days=1)
        return day

    @staticmethod
    def get_nth_weekday_in_month(year, month, weekday, n=1, start=None):
        """Get the nth weekday in a given month. e.g:

        >>> # the 1st monday in Jan 2013
        >>> Calendar.get_nth_weekday_in_month(2013, 1, MON)
        datetime.date(2013, 1, 7)
        >>> # The 2nd monday in Jan 2013
        >>> Calendar.get_nth_weekday_in_month(2013, 1, MON, 2)
        datetime.date(2013, 1, 14)
        """
        day = date(year, month, 1)
        if start:
            day = start
        counter = 0
        while True:
            if day.month != month:
                # Don't forget to break if "n" is too big
                return None
            if day.weekday() == weekday:
                counter += 1
            if counter == n:
                break
            day = day + timedelta(days=1)
        return day

    @staticmethod
    def get_last_weekday_in_month(year, month, weekday):
        """Get the last weekday in a given month. e.g:

        >>> # the last monday in Jan 2013
        >>> Calendar.get_last_weekday_in_month(2013, 1, MON)
        datetime.date(2013, 1, 28)
        """
        day = date(year, month, monthrange(year, month)[1])
        while True:
            if day.weekday() == weekday:
                break
            day = day - timedelta(days=1)
        return day

    @staticmethod
    def get_first_weekday_after(day, weekday):
        """Get the first weekday after a given day. If the day is the same
        weekday, the same day will be returned.

        >>> # the first monday after Apr 1 2015
        >>> Calendar.get_first_weekday_after(date(2015, 4, 1), 0)
        datetime.date(2015, 4, 6)

        >>> # the first tuesday after Apr 14 2015
        >>> Calendar.get_first_weekday_after(date(2015, 4, 14), 1)
        datetime.date(2015, 4, 14)
        """
        day_delta = (weekday - day.weekday()) % 7
        day = day + timedelta(days=day_delta)
        return day


class ChristianMixin(Calendar):
    EASTER_METHOD = None  # to be assigned in the inherited mixin
    include_epiphany = False
    include_clean_monday = False
    include_annunciation = False
    include_ash_wednesday = False
    include_palm_sunday = False
    include_holy_thursday = False
    include_good_friday = False
    include_easter_monday = False
    include_easter_saturday = False
    include_easter_sunday = False
    include_all_saints = False
    include_immaculate_conception = False
    include_christmas = True
    include_christmas_eve = False
    include_ascension = False
    include_assumption = False
    include_whit_sunday = False
    whit_sunday_label = 'Whit Sunday'
    include_whit_monday = False
    whit_monday_label = 'Whit Monday'
    include_corpus_christi = False
    include_boxing_day = False
    boxing_day_label = "Boxing Day"

    def get_ash_wednesday(self, year):
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=46)

    def get_palm_sunday(self, year):
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=7)

    def get_holy_thursday(self, year):
        "Return the date of the last thursday before easter"
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=3)

    def get_good_friday(self, year):
        "Return the date of the last friday before easter"
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=2)

    def get_clean_monday(self, year):
        "Return the clean monday date"
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=48)

    def get_easter_saturday(self, year):
        "Return the Easter Saturday date"
        sunday = self.get_easter_sunday(year)
        return sunday - timedelta(days=1)

    def get_easter_sunday(self, year):
        "Return the date of the easter (sunday) -- following the easter method"
        return easter.easter(year, self.EASTER_METHOD)

    def get_easter_monday(self, year):
        "Return the date of the monday after easter"
        sunday = self.get_easter_sunday(year)
        return sunday + timedelta(days=1)

    def get_ascension_thursday(self, year):
        easter = self.get_easter_sunday(year)
        return easter + timedelta(days=39)

    def get_whit_monday(self, year):
        easter = self.get_easter_sunday(year)
        return easter + timedelta(days=50)

    def get_whit_sunday(self, year):
        easter = self.get_easter_sunday(year)
        return easter + timedelta(days=49)

    def get_corpus_christi(self, year):
        return self.get_easter_sunday(year) + timedelta(days=60)

    def get_variable_days(self, year):  # noqa
        "Return the christian holidays list according to the mixin"
        days = super(ChristianMixin, self).get_variable_days(year)
        if self.include_epiphany:
            days.append((date(year, 1, 6), "Epiphany"))
        if self.include_clean_monday:
            days.append((self.get_clean_monday(year), "Clean Monday"))
        if self.include_annunciation:
            days.append((date(year, 3, 25), "Annunciation"))
        if self.include_ash_wednesday:
            days.append((self.get_ash_wednesday(year), "Ash Wednesday"))
        if self.include_palm_sunday:
            days.append((self.get_palm_sunday(year), "Palm Sunday"))
        if self.include_holy_thursday:
            days.append((self.get_holy_thursday(year), "Holy Thursday"))
        if self.include_good_friday:
            days.append((self.get_good_friday(year), "Good Friday"))
        if self.include_easter_saturday:
            days.append((self.get_easter_saturday(year), "Easter Saturday"))
        if self.include_easter_sunday:
            days.append((self.get_easter_sunday(year), "Easter Sunday"))
        if self.include_easter_monday:
            days.append((self.get_easter_monday(year), "Easter Monday"))
        if self.include_assumption:
            days.append((date(year, 8, 15), "Assumption of Mary to Heaven"))
        if self.include_all_saints:
            days.append((date(year, 11, 1), "All Saints Day"))
        if self.include_immaculate_conception:
            days.append((date(year, 12, 8), "Immaculate Conception"))
        if self.include_christmas:
            days.append((date(year, 12, 25), "Christmas Day"))
        if self.include_christmas_eve:
            days.append((date(year, 12, 24), "Christmas Eve"))
        if self.include_boxing_day:
            days.append((date(year, 12, 26), self.boxing_day_label))
        if self.include_ascension:
            days.append((
                self.get_ascension_thursday(year), "Ascension Thursday"))
        if self.include_whit_monday:
            days.append((self.get_whit_monday(year), self.whit_monday_label))
        if self.include_whit_sunday:
            days.append((self.get_whit_sunday(year), self.whit_sunday_label))
        if self.include_corpus_christi:
            days.append((self.get_corpus_christi(year), "Corpus Christi"))
        return days


class WesternCalendar(Calendar):
    """
    General usage calendar for Western countries.

    (chiefly Europe and Northern America)

    """
    EASTER_METHOD = easter.EASTER_WESTERN
    WEEKEND_DAYS = (SAT, SUN)
    shift_new_years_day = False

    FIXED_HOLIDAYS = (
        (1, 1, 'New year'),
    )

    def get_weekend_days(self):
        "Week-end days are SATurday and SUNday."
        return self.WEEKEND_DAYS

    def get_variable_days(self, year):
        days = super(WesternCalendar, self).get_variable_days(year)
        new_year = date(year, 1, 1)
        if self.shift_new_years_day:
            if new_year.weekday() in self.get_weekend_days():
                days.append((
                    self.find_following_working_day(new_year),
                    "New Year shift"))
        return days


class OrthodoxMixin(ChristianMixin):
    EASTER_METHOD = easter.EASTER_ORTHODOX


class LunarCalendar(Calendar):
    """Calendar including lunar days
    """
    FIXED_HOLIDAYS = (
        (1, 1, 'Lunar new year'),
    )

    @staticmethod
    def lunar(year, month, day):
        return LunarDate(year, month, day).toSolarDate()


class EphemMixin(LunarCalendar):
    def calculate_equinoxes(self, year, timezone='UTC'):
        """ calculate equinox with time zone """

        tz = pytz.timezone(timezone)

        d1 = ephem.next_equinox(str(year))
        d = ephem.Date(str(d1))
        equinox1 = d.datetime() + tz.utcoffset(d.datetime())

        d2 = ephem.next_equinox(d1)
        d = ephem.Date(str(d2))
        equinox2 = d.datetime() + tz.utcoffset(d.datetime())

        return (equinox1.date(), equinox2.date())

    def solar_term(self, year, degrees, timezone='UTC'):
        """
        Returns the date of the solar term for the given longitude
        and the given year.

        Solar terms are used for Chinese and Taiwanese holidays
        (e.g. Qingming Festival in Taiwan).

        More information:
        - https://en.wikipedia.org/wiki/Solar_term
        - https://en.wikipedia.org/wiki/Qingming

        This function is adapted from the following topic:
        https://answers.launchpad.net/pyephem/+question/110832
        """
        twopi = 2 * pi
        tz = pytz.timezone(timezone)

        # Find out the sun's current longitude.

        sun = ephem.Sun(ephem.Date(str(year)))
        current_longitude = sun.hlong - pi

        # Find approximately the right time of year.

        target_longitude = degrees * ephem.degree
        difference = (target_longitude - current_longitude) % twopi
        t0 = ephem.Date(str(year)) + 365.25 * difference / twopi

        # Zero in on the exact moment.

        def f(t):
            sun.compute(t)
            longitude = sun.hlong - pi
            return ephem.degrees(target_longitude - longitude).znorm

        d = ephem.Date(ephem.newton(f, t0, t0 + ephem.minute))
        solar_term = d.datetime() + tz.utcoffset(d.datetime())

        return solar_term.date()


class CalverterMixin(Calendar):
    conversion_method = None
    ISLAMIC_HOLIDAYS = ()

    def __init__(self, *args, **kwargs):
        super(CalverterMixin, self).__init__(*args, **kwargs)
        self.calverter = Calverter()
        if self.conversion_method is None:
            raise NotImplementedError

    def converted(self, year):
        conversion_method = getattr(
            self.calverter, 'jd_to_%s' % self.conversion_method)
        current = date(year, 1, 1)
        days = []
        while current.year == year:
            julian_day = self.calverter.gregorian_to_jd(
                current.year,
                current.month,
                current.day)
            days.append(conversion_method(julian_day))
            current = current + timedelta(days=1)
        return days

    def calverted_years(self, year):
        converted = self.converted(year)
        generator = (y for y, m, d in converted)
        return sorted(list(set(generator)))

    def get_islamic_holidays(self):
        return self.ISLAMIC_HOLIDAYS

    def get_variable_days(self, year):
        warnings.warn('Please take not that, due to arbitrary decisions, '
                      'this Islamic calendar computation may be wrong.')
        days = super(CalverterMixin, self).get_variable_days(year)
        years = self.calverted_years(year)
        conversion_method = getattr(
            self.calverter, '%s_to_jd' % self.conversion_method)
        for month, day, label in self.get_islamic_holidays():
                for y in years:
                    jd = conversion_method(y, month, day)
                    g_year, g_month, g_day = self.calverter.jd_to_gregorian(jd)
                    if g_year == year:
                        holiday = date(g_year, g_month, g_day)
                        days.append((holiday, label))
        return days


class IslamicMixin(CalverterMixin):
    conversion_method = 'islamic'
    include_prophet_birthday = False
    include_day_after_prophet_birthday = False
    include_start_ramadan = False
    include_eid_al_fitr = False
    length_eid_al_fitr = 1
    include_eid_al_adha = False
    length_eid_al_adha = 1
    include_day_of_sacrifice = False
    include_day_of_sacrifice_label = "Eid al-Adha"
    include_islamic_new_year = False
    include_laylat_al_qadr = False

    def get_islamic_holidays(self):
        """Return a list of Islamic (month, day, label) for islamic holidays.
        Please take note that these dates must be expressed using the Islamic
        Calendar"""
        days = list(super(IslamicMixin, self).get_islamic_holidays())

        if self.include_islamic_new_year:
            days.append((1, 1, "Islamic New Year"))
        if self.include_prophet_birthday:
            days.append((3, 12, "Prophet's Birthday"))
        if self.include_day_after_prophet_birthday:
            days.append((3, 13, "Day after Prophet's Birthday"))
        if self.include_start_ramadan:
            days.append((9, 1, "Start of ramadan"))
        if self.include_eid_al_fitr:
            for x in range(self.length_eid_al_fitr):
                days.append((10, x + 1, "Eid al-Fitr"))
        if self.include_eid_al_adha:
            for x in range(self.length_eid_al_adha):
                days.append((12, x + 10, "Eid al-Adha"))
        if self.include_day_of_sacrifice:
            days.append((12, 10, self.include_day_of_sacrifice_label))
        if self.include_laylat_al_qadr:
            warnings.warn("The Islamic holiday named Laylat al-Qadr is decided"
                          " by the religious authorities. It is not possible"
                          " to compute it. You'll have to add it manually.")
        return tuple(days)


class JalaliMixin(CalverterMixin):
    conversion_method = 'jalali'



# -*- coding: utf-8 -*-
from datetime import date, timedelta
from workalendar.core import WesternCalendar, ChristianMixin, OrthodoxMixin
from workalendar.core import THU, MON, FRI, SAT


class CzechRepublic(WesternCalendar, ChristianMixin):
    "Czech Republic"
    include_easter_monday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 1, "Restoration Day of the Independent Czech State"),
        (5, 1, "Labour Day"),
        (5, 8, "Liberation Day"),
        (7, 5, "Saints Cyril and Methodius Day"),
        (7, 6, "Jan Hus Day"),
        (9, 28, "St. Wenceslas Day (Czech Statehood Day)"),
        (10, 28, "Independent Czechoslovak State Day"),
        (11, 17, "Struggle for Freedom and Democracy Day"),
        (12, 24, "Christmas Eve"),
        (12, 26, "St. Stephen's Day (The Second Christmas Day)"),
    )


class Denmark(WesternCalendar, ChristianMixin):
    "Denmark"
    include_palm_sunday = True
    include_holy_thursday = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_whit_monday = True
    whit_monday_label = "Pentecost Monday"
    include_boxing_day = True
    boxing_day_label = "Second Day of Christmas"
    include_christmas_eve = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (6, 5, "Constitution Day"),
        (12, 31, "New Year's Eve")
    )

    def get_store_bededag(self, year):  # 'great prayer day'
        easter_sunday = self.get_easter_sunday(year)
        return easter_sunday + timedelta(days=26)

    def get_variable_days(self, year):
        days = super(Denmark, self).get_variable_days(year)
        days.append((self.get_store_bededag(year), "Store Bededag"))
        return days


class Slovakia(WesternCalendar, ChristianMixin):
    "Slovakia"
    include_epiphany = True
    include_easter_monday = True
    include_good_friday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 1, "Day of the Establishment of the Slovak Republic"),
        (5, 1, "Labour Day"),
        (5, 8, "Liberation Day"),
        (7, 5, "Saints Cyril and Methodius Day"),
        (8, 29, "Slovak National Uprising anniversary"),
        (9, 1, "Day of the Constitution of the Slovak Republic"),
        (9, 15, "Day of Blessed Virgin Mary, patron saint of Slovakia"),
        (11, 1, "All Saints’ Day"),
        (11, 17, "Struggle for Freedom and Democracy Day"),
        (12, 24, "Christmas Eve"),
        (12, 25, "Christmas Day"),
        (12, 26, "St. Stephen's Day (The Second Christmas Day)"),
    )


class Sweden(WesternCalendar, ChristianMixin):
    "Sweden"
    include_epiphany = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost"
    # Christmas Eve is not a holiday but not a work day either
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "Second Day of Christmas"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (6, 6, "National Day"),
        # New Year's Eve is not a holiday but not a work day either
        (12, 31, "New Year's Eve")
    )

    # Midsummer Eve is not a holiday but not a work day either
    def get_midsummer_eve(self, year):
        date_eve = Sweden.get_nth_weekday_in_month(
            year, 6, FRI, start=date(year, 6, 19))
        return date_eve

    def get_midsummer_day(self, year):
        date_eve = Sweden.get_nth_weekday_in_month(
            year, 6, SAT, start=date(year, 6, 20))
        return date_eve

    def get_variable_all_saints(self, year):
        all_saints = date(year, 10, 31)
        if all_saints.weekday() != SAT:
            all_saints = Sweden.get_nth_weekday_in_month(
                year, 11, SAT)
        return all_saints

    def get_variable_days(self, year):
        days = super(Sweden, self).get_variable_days(year)
        days.append((self.get_midsummer_day(year), "Midsummer's Day"))
        days.append((self.get_midsummer_eve(year), "Midsummer's Eve"))
        days.append((self.get_variable_all_saints(year), "All Saints"))
        return days


class Finland(WesternCalendar, ChristianMixin):
    "Finland"
    include_epiphany = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    whit_sunday_label = 'Pentecost'
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "St. Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (12, 6, "Independence Day"),
    )

    def get_midsummer_eve(self, year):
        date_eve = Finland.get_nth_weekday_in_month(
            year, 6, FRI, start=date(year, 6, 19))
        return date_eve

    def get_midsummer_day(self, year):
        date_eve = Finland.get_nth_weekday_in_month(
            year, 6, SAT, start=date(year, 6, 20))
        return date_eve

    def get_variable_all_saints(self, year):
        all_saints = date(year, 10, 31)
        if all_saints.weekday() != SAT:
            all_saints = Finland.get_nth_weekday_in_month(
                year, 11, SAT)
        return all_saints

    def get_variable_days(self, year):
        days = super(Finland, self).get_variable_days(year)
        days.append((self.get_midsummer_eve(year), "Midsummer's Eve"))
        days.append((self.get_midsummer_day(year), "Midsummer's Day"))
        days.append((self.get_variable_all_saints(year), "All Saints"))
        return days


class France(WesternCalendar, ChristianMixin):
    "France"
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_all_saints = True
    include_assumption = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (5, 8, "Victory in Europe Day"),
        (7, 14, "Bastille Day"),
        (11, 11, "Armistice Day"),
    )


class Luxembourg(WesternCalendar, ChristianMixin):
    "Luxembourg"
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_all_saints = True
    include_assumption = True
    include_boxing_day = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (6, 23, "Luxembourg National Holiday"),
    )


class Netherlands(WesternCalendar, ChristianMixin):
    "Netherlands"
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    include_whit_monday = True
    include_boxing_day = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 5, "Liberation Day"),
        (12, 31, "New Year's Eve"),
    )

    def get_kingsday(self, year):
        """27 April unless this is a Sunday in which case it is the 26th """
        if (date(year, 4, 27).weekday() != 6):
            return date(year, 4, 27)
        return date(year, 4, 26)

    def get_variable_days(self, year):
        days = super(Netherlands, self).get_variable_days(year)
        days += [
            (
                self.get_kingsday(year),
                "King's Day")
        ]
        return days


class FranceAlsaceMoselle(France):
    "France Alsace/Moselle"
    include_good_friday = True
    include_boxing_day = True


class Greece(OrthodoxMixin, WesternCalendar):
    "Greece"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 25, "Independence Day"),
        (5, 1, "Labour Day"),
        (10, 28, "Ohi Day"),
    )
    include_epiphany = True
    include_clean_monday = True
    include_annunciation = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost"
    include_whit_monday = True
    include_assumption = True
    include_boxing_day = True
    boxing_day_label = "Glorifying Mother of God"


class Hungary(WesternCalendar, ChristianMixin):
    "Hungary"
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_whit_monday = True
    whit_monday_label = "Pentecost Monday"
    include_boxing_day = True
    boxing_day_label = "Second Day of Christmas"
    include_all_saints = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (3, 15, "National Day"),
        (5, 1, "Labour Day"),
        (8, 20, "St Stephen's Day"),
        (10, 23, "National Day"),
    )


class Iceland(WesternCalendar, ChristianMixin):
    "Iceland"
    include_holy_thursday = True
    include_good_friday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_christmas_eve = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (6, 17, "Icelandic National Day"),
        (12, 31, "New Year's Eve"),
    )

    def get_first_day_of_summer(self, year):
        """It's the first thursday *after* April, 18th.
        If April the 18th is a thursday, then it jumps to the 24th.
        """
        return WesternCalendar.get_nth_weekday_in_month(
            year, 4, THU,
            start=date(year, 4, 19))

    def get_variable_days(self, year):
        days = super(Iceland, self).get_variable_days(year)
        days += [
            (
                self.get_first_day_of_summer(year),
                "First day of summer"),
            (
                Iceland.get_nth_weekday_in_month(year, 8, MON),
                "Commerce Day"),
        ]
        return days


class Italy(WesternCalendar, ChristianMixin):
    "Italy"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (4, 25, "Liberation Day"),
        (5, 1, "International Workers' Day"),
        (6, 2, "Republic Day"),
    )
    include_immaculate_conception = True
    include_epiphany = True
    include_easter_monday = True
    include_assumption = True
    include_all_saints = True
    include_assumption = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"


class Norway(WesternCalendar, ChristianMixin):
    "Norway"
    include_holy_thursday = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_whit_sunday = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (5, 17, "Constitution Day"),
    )


class Poland(WesternCalendar, ChristianMixin):
    "Poland"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 6, 'Trzech Kroli'),
        (5, 1, 'Labour Day'),
        (5, 3, 'Constitution Day'),
        (11, 11, 'Independence Day'),
    )
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_corpus_christi = True
    include_assumption = True
    include_all_saints = True
    include_boxing_day = True


class UnitedKingdom(WesternCalendar, ChristianMixin):
    "United Kingdom"
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_boxing_day = True
    shift_new_years_day = True

    def get_early_may_bank_holiday(self, year):
        return (
            UnitedKingdom.get_nth_weekday_in_month(year, 5, MON),
            "Early May Bank Holiday"
        )

    def get_spring_bank_holiday(self, year):
        return (
            UnitedKingdom.get_last_weekday_in_month(year, 5, MON),
            "Spring Bank Holiday"
        )

    def get_late_summer_bank_holiday(self, year):
        return (
            UnitedKingdom.get_last_weekday_in_month(year, 8, MON),
            "Late Summer Bank Holiday"
        )

    def get_variable_days(self, year):
        days = super(UnitedKingdom, self).get_variable_days(year)
        days.append(self.get_early_may_bank_holiday(year))
        days.append(self.get_spring_bank_holiday(year))
        days.append(self.get_late_summer_bank_holiday(year))
        # Boxing day & XMas shift
        christmas = date(year, 12, 25)
        boxing_day = date(year, 12, 26)
        if christmas.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(christmas)
            days.append((shift, "Christmas Shift"))
            days.append((shift + timedelta(days=1), "Boxing Day Shift"))
        elif boxing_day.weekday() in self.get_weekend_days():
            shift = self.find_following_working_day(boxing_day)
            days.append((shift, "Boxing Day Shift"))
        return days


class UnitedKingdomNorthernIreland(UnitedKingdom):
    "Northern Ireland (UK)"
    def get_variable_days(self, year):
        days = super(UnitedKingdomNorthernIreland, self) \
            .get_variable_days(year)
        # St Patrick's day
        st_patrick = date(year, 3, 17)
        days.append((st_patrick, "Saint Patrick's Day"))
        if st_patrick.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(st_patrick),
                "Saint Patrick substitute"))

        # Battle of boyne
        battle_of_boyne = date(year, 7, 12)
        days.append((battle_of_boyne, "Battle of the Boyne"))
        if battle_of_boyne.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(battle_of_boyne),
                "Battle of the Boyne substitute"))
        return days


class EuropeanCentralBank(WesternCalendar, ChristianMixin):
    "European Central Bank"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (12, 26, "St. Stephen's Day"),
    )

    include_good_friday = True
    include_easter_monday = True


class Belgium(WesternCalendar, ChristianMixin):
    "Belgium"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (7, 21, "National Day"),
        (11, 11, "Armistice of 1918"),
    )

    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_assumption = True
    include_all_saints = True


class Germany(WesternCalendar, ChristianMixin):
    "Germany"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, "Labour Day"),
        (10, 3, "Day of German Unity"),
    )

    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_good_friday = True
    include_boxing_day = True
    boxing_day_label = "Second Christmas Day"


class BadenWurttemberg(Germany):
    "Baden-Württemberg"

    include_epiphany = True
    include_corpus_christi = True
    include_all_saints = True


class Bavaria(Germany):
    "Bavaria"

    include_epiphany = True
    include_corpus_christi = True
    include_all_saints = True
    include_assumption = True


class Berlin(Germany):
    "Berlin"


class Brandenburg(Germany):
    "Brandenburg"

    FIXED_HOLIDAYS = Germany.FIXED_HOLIDAYS + (
        (10, 31, "Reformation Day"),
    )


class Bremen(Germany):
    "Bremen"


class Hamburg(Germany):
    "Hamburg"


class Hesse(Germany):
    "Hesse"

    include_corpus_christi = True


class MecklenburgVorpommern(Germany):
    "Mecklenburg-Vorpommern"

    FIXED_HOLIDAYS = Germany.FIXED_HOLIDAYS + (
        (10, 31, "Reformation Day"),
    )


class LowerSaxony(Germany):
    "Lower Saxony"


class NorthRhineWestphalia(Germany):
    "North Rhine-Westphalia"

    include_corpus_christi = True
    include_all_saints = True


class RhinelandPalatinate(Germany):
    "Rhineland-Palatinate"

    include_corpus_christi = True
    include_all_saints = True


class Saarland(Germany):
    "Saarland"

    include_corpus_christi = True
    include_assumption = True
    include_all_saints = True


class Saxony(Germany):
    "Saxony"

    FIXED_HOLIDAYS = Germany.FIXED_HOLIDAYS + (
        (10, 31, "Reformation Day"),
    )

    def get_repentance_day(self, year):
        "Wednesday before November 23"
        day = date(year, 11, 23)
        while day.weekday() != 2:  # 2=Wednesday
            day -= timedelta(days=1)
        return (day, "Repentance Day")

    def get_variable_days(self, year):
        days = super(Germany, self).get_variable_days(year)
        days.append(self.get_repentance_day(year))
        return days


class SaxonyAnhalt(Germany):
    "Saxony-Anhalt"

    FIXED_HOLIDAYS = Germany.FIXED_HOLIDAYS + (
        (10, 31, "Reformation Day"),
    )

    include_epiphany = True


class SchleswigHolstein(Germany):
    "Schleswig-Holstein"


class Thuringia(Germany):
    "Thuringia"

    FIXED_HOLIDAYS = Germany.FIXED_HOLIDAYS + (
        (10, 31, "Reformation Day"),
    )


class Portugal(WesternCalendar, ChristianMixin):
    "Portugal"
    include_good_friday = True
    include_easter_sunday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (4, 25, "Dia da Liberdade"),
        (5, 1, "Dia do Trabalhador"),
        (6, 10, "Dia de Portugal"),
        (8, 15, "Assunção de Nossa Senhora"),
        (12, 8, "Imaculada Conceição"),
    )

    def get_variable_entrudo(self, year):
        easter_sunday = self.get_easter_sunday(year)
        return easter_sunday - timedelta(days=47)

    def get_variable_days(self, year):
        days = super(Portugal, self).get_variable_days(year)
        days.append((self.get_variable_entrudo(year), "Entrudo"))

        return days


class Spain(WesternCalendar, ChristianMixin):
    "Spain"
    include_epiphany = True
    include_immaculate_conception = True
    include_good_friday = True
    include_assumption = True
    include_all_saints = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 1, u"Día del trabajador"),
        (10, 12, u"Fiesta nacional de España"),
        (12, 6, u"Día de la Constitución Española")
    )


class Slovenia(WesternCalendar, ChristianMixin):
    "Slovenia"
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    include_assumption = True
    include_christmas = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (2, 8, "Preseren Day, the Slovenian Cultural Holiday"),
        (4, 27, "Day of Uprising Against Occupation"),
        (5, 1, "Labour Day"),
        (5, 2, "Labour Day"),
        (6, 25, "Statehood Day"),
        (10, 31, "Reformation Day"),
        (11, 1, "Day of Remembrance of the Dead"),
        (12, 26, "Independence and Unity Day"),
    )


class Switzerland(WesternCalendar, ChristianMixin):
    "Switzerland"

    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    include_whit_monday = True
    include_christmas = True
    include_boxing_day = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (1, 2, "Berchtold's Day"),
        (5, 1, "Labour Day"),
        (8, 1, "National Holiday"),
    )



# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from datetime import date, timedelta
from workalendar.core import WesternCalendar, ChristianMixin, Calendar
from workalendar.core import SUN, MON, TUE, WED, THU, FRI, SAT

NONE, NEAREST_WEEKDAY, MONDAY = range(3)


class UnitedStates(WesternCalendar, ChristianMixin):
    "United States of America"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (7, 4, 'Independence Day'),
        (11, 11, 'Veterans Day'),
    )

    @staticmethod
    def is_presidential_year(year):
        return (year % 4) == 0

    def get_variable_days(self, year):
        # usual variable days
        days = super(UnitedStates, self).get_variable_days(year)
        days += [
            (UnitedStates.get_nth_weekday_in_month(year, 1, MON, 3),
                'Martin Luther King, Jr. Day'),

            (UnitedStates.get_nth_weekday_in_month(year, 2, MON, 3),
                "Washington's Birthday"),

            (UnitedStates.get_last_weekday_in_month(year, 5, MON),
                "Memorial Day"),

            (UnitedStates.get_nth_weekday_in_month(year, 9, MON),
                "Labor Day"),

            (UnitedStates.get_nth_weekday_in_month(year, 10, MON, 2),
                "Colombus Day"),

            (UnitedStates.get_nth_weekday_in_month(year, 11, THU, 4),
                "Thanksgiving Day"),
        ]
        # Inauguration day
        if UnitedStates.is_presidential_year(year - 1):
            inauguration_day = date(year, 1, 20)
            if inauguration_day.weekday() == SUN:
                inauguration_day = date(year, 1, 21)
            days.append((inauguration_day, "Inauguration Day"))
        return days


class FloatToNearestWeekdayMixin(Calendar):
    "Float Saturady to Friday, Sunday to Monday"
    def float(self, holidays, year=0):
        new_holidays = []
        holiday_lookup = [x[0] for x in holidays]
        for holiday in holidays:
            if holiday[0].weekday() == SAT:
                new_holidays.append((holiday[0] - timedelta(days=1),
                                     holiday[1] + " (Observed)"))
            elif holiday[0].weekday() == SUN:
                new_holidays.append((holiday[0] + timedelta(days=1),
                                     holiday[1] + " (Observed)"))

        if year > 0 and date(year + 1, 1, 1).weekday() == SAT:
            new_holidays.append((date(year, 12, 31,),
                                 "New Years Day (Observed)"))

        year = holiday_lookup[0].year
        if (date(year, 12, 25) in holiday_lookup and
                date(year, 12, 24) in holiday_lookup and
                date(year, 12, 25).weekday() == SAT):
            new_holidays.append((date(year, 12, 23),
                                 "Christmas Eve (Observed)"))
        if (date(year, 12, 25) in holiday_lookup and
                date(year, 12, 24) in holiday_lookup and
                date(year, 12, 25).weekday() == MON):
            new_holidays.append((date(year, 12, 26),
                                 "Christmas Eve (Observed)"))
            new_holidays.append((date(year, 12, 27),
                                 "Christmas Day (Observed)"))
        return holidays + new_holidays


class WashingtonsBirthdayInDecemberMixin(Calendar):
    """Floating observance, to give long weekend at christmas"""
    def get_washington_birthday(self, year,
                                label="Washington's Birthday (Observed)"):
        christmas_day = date(year, 12, 25).weekday()
        if christmas_day == MON:
            day = (date(year, 12, 26), label)  # TUE
        elif christmas_day == TUE:
            day = (date(year, 12, 24), label)  # MON
        elif christmas_day == WED:
            day = (date(year, 12, 24), label)  # TUE
        elif christmas_day == THU:
            day = (date(year, 12, 26), label)  # FRI
        elif christmas_day == FRI:
            day = (date(year, 12, 24), label)  # THU
        elif christmas_day == SAT:
            day = (date(year, 12, 23), label)  # THU
        else:  # christmas_day == SUN:
            day = (date(year, 12, 23), label)  # FRI
        return day


class CesarChavezDayMixin(Calendar):
    """31st of March, float to 1st April if Monday"""
    def get_chavez_day(self, year):
        days = [(date(year, 3, 31), "Cesar Chavez Day")]
        if date(year, 3, 31).weekday() == SUN:
            days += [(date(year, 4, 1), "Cesar Chavez Day (Observed)")]
        return days


class MardiGrasMixin(ChristianMixin):
    """Tuesday before Ash Wednesday"""
    def get_mardis_gras(self, year):
        sunday = self.get_easter_sunday(year)
        return (sunday - timedelta(days=47), "Mardi Gras")


class DayAfterChristmasNoFloatMixin(Calendar):
    """26th of December - but doesn't ever float"""
    def get_day_after_christmas(self, year):
        days = (date(year, 12, 26), "Day After Christmas")
        return days


class PatriotsDayMixin(Calendar):
    """3rd Monday of April"""
    def get_patriots_day(self, year):
        days = (self.get_nth_weekday_in_month(year, 4, MON, 3),
                "Patriots Day")
        return days


class ConfederateMemorialDayMixin(Calendar):
    """4th Monday of April"""
    def get_confederate_day(self, year):
        return (Alabama.get_nth_weekday_in_month(year, 4, MON, 4),
                "Confederate Day")


class ThanksgivingFridayMixin(Calendar):
    "4th Friday in November"
    def get_thanksgiving_friday(self, year, label="Thanksgiving Friday"):
        return (self.get_nth_weekday_in_month(year, 11, FRI, 4), label)


class Alabama(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
              ConfederateMemorialDayMixin):
    "Alabama"
    def get_variable_days(self, year):
        days = super(Alabama, self).get_variable_days(year)
        days = super(Alabama, self).float(days)
        days = days + [self.get_confederate_day(year)]
        days = days + [(Alabama.get_nth_weekday_in_month(year, 6, MON, 1),
                        "Jefferson Davis Birthday")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Alabama, self).get_fixed_holidays(year)
        days = super(Alabama, self).float(days, year)
        return days


class Alaska(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Alaska"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (10, 18, 'Alaska Day'),
    )

    def get_variable_days(self, year):
        days = super(Alaska, self).get_variable_days(year)
        days = super(Alaska, self).float(days)
        days = days + [(Alabama.get_last_weekday_in_month(year, 3, MON),
                        "Seward's Day")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Alaska, self).get_fixed_holidays(year)
        days = super(Alaska, self).float(days, year)
        return days


class Arizona(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Arizona"""
    def get_variable_days(self, year):
        days = super(Arizona, self).get_variable_days(year)
        days = super(Arizona, self).float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Arizona, self).get_fixed_holidays(year)
        days = super(Arizona, self).float(days, year)
        return days


class Arkansas(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Arkansas"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Arkansas, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Arkansas, self).get_fixed_holidays(year)
        days = super(Arkansas, self).float(days, year)
        return days


class California(UnitedStates, WesternCalendar, ThanksgivingFridayMixin,
                 FloatToNearestWeekdayMixin, CesarChavezDayMixin):
    """California"""
    def get_variable_days(self, year):
        days = super(California, self).get_variable_days(year)
        days += [self.get_thanksgiving_friday(year)]
        days += [(self.get_nth_weekday_in_month(year, 10, MON, 2),
                  "Indingenous People's Day")]
        days += self.get_chavez_day(year)
        return days

    def get_fixed_holidays(self, year):
        days = super(California, self).get_fixed_holidays(year)
        days = super(California, self).float(days, year)
        return days


class Colorado(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               CesarChavezDayMixin):
    """Colorado"""

    def get_variable_days(self, year):
        days = super(Colorado, self).get_variable_days(year)
        days += self.get_chavez_day(year)
        return days

    def get_fixed_holidays(self, year):
        days = super(Colorado, self).get_fixed_holidays(year)
        days = super(Colorado, self).float(days, year)
        return days


class Connecticut(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Connecticut"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (2, 12, "Lincoln's Birthday"),
    )

    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Connecticut, self).get_variable_days(year)
        days = super(Connecticut, self).float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Connecticut, self).get_fixed_holidays(year)
        days = super(Connecticut, self).float(days, year)
        return days


class Delaware(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Delaware"""

    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Delaware, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Delaware, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Florida(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Florida"""
    def get_variable_days(self, year):
        days = super(Florida, self).get_variable_days(year)
        days = super(Florida, self).float(days)
        days = days + [(Florida.get_nth_weekday_in_month(year, 11, FRI, 4),
                        "Friday after Thanksgiving")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Florida, self).get_fixed_holidays(year)
        days = super(Florida, self).float(days, year)
        return days


class Georgia(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
              WashingtonsBirthdayInDecemberMixin, ConfederateMemorialDayMixin):
    """Georgia"""
    def get_variable_days(self, year):
        days = super(Georgia, self).get_variable_days(year)
        days = super(Georgia, self).float(days)
        days += [self.get_confederate_day(year)]
        days += [(Georgia.get_nth_weekday_in_month(year, 11, FRI, 4),
                  "Robert E. Lee's Birthday (Observed)")]
        days += [self.get_washington_birthday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Georgia, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Hawaii(UnitedStates, WesternCalendar):
    """Hawaii"""
    pass


class Idaho(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Idaho"""
    def get_variable_days(self, year):
        days = super(Idaho, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Idaho, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Illinois(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Illinois"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (2, 12, "Lincoln's Birthday"),
    )

    def get_variable_days(self, year):
        days = super(Illinois, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Illinois, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Indiana(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
              WashingtonsBirthdayInDecemberMixin, ThanksgivingFridayMixin):
    """Indiana"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Indiana, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_washington_birthday(year)]
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Indiana, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Iowa(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
           ThanksgivingFridayMixin):
    """Iowa"""
    def get_variable_days(self, year):
        days = super(Iowa, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Iowa, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Kansas(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
             ThanksgivingFridayMixin, DayAfterChristmasNoFloatMixin):
    """Kansas"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Kansas, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [self.get_day_after_christmas(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Kansas, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Kentucky(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin, DayAfterChristmasNoFloatMixin):
    """Kentucky"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Kentucky, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [self.get_day_after_christmas(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Kentucky, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Louisiana(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                MardiGrasMixin):
    """Louisiana"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Louisiana, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_mardis_gras(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Louisiana, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Maine(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
            PatriotsDayMixin, ThanksgivingFridayMixin):
    """Maine"""

    def get_variable_days(self, year):
        days = super(Maine, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_patriots_day(year)]
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Maine, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Maryland(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Maryland"""
    def get_variable_days(self, year):
        days = super(Maryland, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Maryland, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Massachusetts(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                    PatriotsDayMixin):
    """Massachusetts"""
    def get_variable_days(self, year):
        days = super(Massachusetts, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_patriots_day(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Massachusetts, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Michigan(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Michigan"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Michigan, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Michigan, self).get_fixed_holidays(year)
        days += [(date(year, 12, 31), "New Years Eve")]
        days = self.float(days, year)
        return days


class Minnesota(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                ThanksgivingFridayMixin):
    """Minnesota"""
    def get_variable_days(self, year):
        days = super(Minnesota, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Minnesota, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Mississippi(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                  ThanksgivingFridayMixin, ConfederateMemorialDayMixin):
    """Mississippi"""
    def get_variable_days(self, year):
        days = super(Mississippi, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_confederate_day(year)]
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Mississippi, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Missouri(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Missouri"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (2, 12, "Lincoln's Birthday"),
        (5, 8, "Truman Day"),
    )

    def get_variable_days(self, year):
        days = super(Missouri, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Missouri, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Montana(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Montana"""

    def get_variable_days(self, year):
        days = super(Montana, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Montana, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Nebraska(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Nebraska"""
    def get_variable_days(self, year):
        days = super(Nebraska, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [(self.get_last_weekday_in_month(year, 4, FRI),
                  "Arbor Day")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Nebraska, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Nevada(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
             ThanksgivingFridayMixin):
    """Nevada"""
    def get_variable_days(self, year):
        days = super(Nevada, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [(self.get_last_weekday_in_month(year, 10, FRI),
                  "Nevada Day")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Nevada, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NewHampshire(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                   ThanksgivingFridayMixin):
    """NewHampshire"""
    def get_variable_days(self, year):
        days = super(NewHampshire, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(NewHampshire, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NewJersey(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """NewJersey"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(NewJersey, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(NewJersey, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NewMexico(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                ThanksgivingFridayMixin):
    """NewMexico"""
    def get_variable_days(self, year):
        days = super(NewMexico, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(NewMexico, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NewYork(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """NewYork"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (2, 12, "Lincoln's Birthday"),
    )

    def get_variable_days(self, year):
        days = super(NewYork, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(NewYork, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NorthCarolina(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                    ThanksgivingFridayMixin, DayAfterChristmasNoFloatMixin):
    """NorthCarolina"""
    include_good_friday = True
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(NorthCarolina, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [self.get_day_after_christmas(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(NorthCarolina, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class NorthDakota(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """NorthDakota"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(NorthDakota, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(NorthDakota, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Ohio(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Ohio"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (12, 1, "Rosa Parks Day"),
    )

    def get_variable_days(self, year):
        days = super(Ohio, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Ohio, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Oklahoma(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin):
    """Oklahoma"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Oklahoma, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Oklahoma, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Oregon(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Oregon"""

    def get_variable_days(self, year):
        days = super(Oregon, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Oregon, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Pennsylvania(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                   ThanksgivingFridayMixin):
    """Pennsylvania"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Pennsylvania, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Pennsylvania, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class RhodeIsland(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """RhodeIsland"""

    def get_variable_days(self, year):
        days = super(RhodeIsland, self).get_variable_days(year)
        days = self.float(days)
        days += [(self.get_nth_weekday_in_month(year, 8, MON, 2),
                  "Victory Day")]
        return days

    def get_fixed_holidays(self, year):
        days = super(RhodeIsland, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class SouthCarolina(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                    ThanksgivingFridayMixin, DayAfterChristmasNoFloatMixin):
    """SouthCarolina"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (5, 10, "Confederate Memorial Day"),
    )
    include_good_friday = True
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(SouthCarolina, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [self.get_day_after_christmas(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(SouthCarolina, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class SouthDakota(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """SouthDakota"""
    def get_variable_days(self, year):
        days = super(SouthDakota, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(SouthDakota, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Tennessee(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                ThanksgivingFridayMixin):
    """Tennessee"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Tennessee, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Tennessee, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Texas(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
            CesarChavezDayMixin):
    """Texas"""
    include_good_friday = True

    def get_variable_days(self, year):
        days = super(Texas, self).get_variable_days(year)
        days = self.float(days)
        days += self.get_chavez_day(year)
        return days

    def get_fixed_holidays(self, year):
        days = super(Texas, self).get_fixed_holidays(year)
        days = self.float(days, year)
        days += [(date(year, 1, 19), "Confederate Heroes Day")]
        days += [(date(year, 3, 2), "Texas Independence Day")]
        days += [(date(year, 4, 21), "San Jacinto Day")]
        days += [(date(year, 6, 19), "Emancipation Day")]
        days += [(date(year, 8, 27), "Lyndon B. Jonhson Day")]
        return days


class Utah(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Utah"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (7, 24, "Pioneer Day"),
    )

    def get_variable_days(self, year):
        days = super(Utah, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Utah, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Vermont(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Vermont"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (8, 16, "Bennington Battle Day"),
    )

    def get_variable_days(self, year):
        days = super(Vermont, self).get_variable_days(year)
        days = self.float(days)
        days += [(self.get_nth_weekday_in_month(year, 3, TUE, 1),
                  "Town Meeting Day")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Vermont, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class Virginia(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
               ThanksgivingFridayMixin, DayAfterChristmasNoFloatMixin):
    """Virginia"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Virginia, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [(self.get_nth_weekday_in_month(year, 1, FRI, 3),
                  "Lee-Jackson Day")]
        days += [(self.get_nth_weekday_in_month(year, 11, WED, 4),
                  "Additional Thanksgiving Holiday")]
        return days

    def get_fixed_holidays(self, year):
        days = super(Virginia, self).get_fixed_holidays(year)
        days = self.float(days, year)
        days += [self.get_day_after_christmas(year)]
        return days


class Washington(UnitedStates, WesternCalendar, ThanksgivingFridayMixin,
                 FloatToNearestWeekdayMixin):
    """Washington"""
    def get_variable_days(self, year):
        days = super(Washington, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Washington, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days


class WestVirginia(UnitedStates, WesternCalendar, ThanksgivingFridayMixin,
                   FloatToNearestWeekdayMixin):
    """WestVirginia"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(WestVirginia, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        days += [(date(year, 6, 20), "West Virgina Day")]
        if date(year, 6, 20).weekday() == SUN:
            days += [(date(year, 6, 21), "West Virgina Day (Observed)")]

        return days

    def get_fixed_holidays(self, year):
        days = super(WestVirginia, self).get_fixed_holidays(year)
        days = self.float(days, year)
        days += [(date(year, 12, 31), "New Years Eve")]
        return days


class Wisconsin(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin,
                ThanksgivingFridayMixin):
    """Wisconsin"""
    include_christmas_eve = True

    def get_variable_days(self, year):
        days = super(Wisconsin, self).get_variable_days(year)
        days = self.float(days)
        days += [self.get_thanksgiving_friday(year)]
        return days

    def get_fixed_holidays(self, year):
        days = super(Wisconsin, self).get_fixed_holidays(year)
        days += [(date(year, 12, 31), "New Years Eve")]
        days = self.float(days, year)
        return days


class Wyoming(UnitedStates, WesternCalendar, FloatToNearestWeekdayMixin):
    """Wyoming"""
    def get_variable_days(self, year):
        days = super(Wyoming, self).get_variable_days(year)
        days = self.float(days)
        return days

    def get_fixed_holidays(self, year):
        days = super(Wyoming, self).get_fixed_holidays(year)
        days = self.float(days, year)
        return days
